<?php

namespace HCM\Geral\Entity;
use HCM\Util\TkEntity;
use HCM\Util\Repositories;

class HierarquiaorgEntity extends TkEntity
{
    public function __construct($entityManager, $hcmParameters){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        parent::__construct(null,$entityManager);
    }
    
    public function retornaHierarquiaPorTipo($nrOrg, $nrTipoHierarquia){
        $resultado = $this->entityManager->getRepository(Repositories::HIERARQUIAORG)->findBy(array('nrorg' => $nrOrg,
            'nrtipohierarqorg' => $nrTipoHierarquia));
        return $resultado;
    }


    public function carregaHierarquiasPorTipo($nrOrg, $nrTipoHierarquia){
        $itens  = $this->getItems();
        if (count($itens) === 0) {
            $itens = $this->retornaHierarquiaPorTipo($nrOrg, $nrTipoHierarquia);
            if (count($itens) > 0) {
                foreach ($itens as $item){
                    $this->addItem($item);
                }
            } else {
                throw new \Exception("Tipo de Hierarquia não encontrada para organização ".$nrOrg);
            }
        }
    }


    /**
     * Procura a hierarquia atraves da estrutura relacionada.
     *
     * @param integer $nr_estrutra Numero da estrutura relacionada (filha).
     * @param array   $hierarquias Lista de hierarquias.
     *
     * @return HierarquiaorgEntity
     *
     * @author alessandrobartels -
     *         alessandrobartels - 05/12/2012 - As hierarquias nao sao mais passadas como parametro.
     *                                          Sao carregadas na lista.
     */
    public function procuraHierarquiaPorEstruturaRel($nr_estrutra, array $hierarquias){

        foreach ($hierarquias as $hierarquia) {
            if ((int)$hierarquia->getNrestruturamrel() === (int)$nr_estrutra) {
                return $hierarquia;
            }
        }

        return null;
    }


    /**
     * Determina estrutura superior.
     *
     * @param integer $nrEstrutura Numero da estrutura relacionada.
     * @param array   $hierarquias Lista de hierarquias.
     *
     * @return integer | null
     *
     * @author alessandrobartels - 05/01/2012
     */
    public function determinaEstruturaSuperior($nrEstrutura, array $hierarquias){
        $hierarquia = $this->procuraHierarquiaPorEstruturaRel($nrEstrutura, $hierarquias);

        if (is_object($hierarquia) === true) {
            $nrestrutura = $hierarquia->getNrestruturamsup();
            if (empty($nrestrutura)){
                return $hierarquia->getNrestruturamrel();
            } else {
                return $hierarquia->getNrestruturamsup();
            }

        } else {
            return null;
        }
    }

    public function setHierarquia($nrorg, $nrtipohierarqorg, $nrestruturamsup, $nrestruturamrel, $dtinivigencia) {
        $this->setNrorg($nrorg);
        $this->setNrtipohierarqorg($nrtipohierarqorg);
        $this->setNrestruturamsup($nrestruturamsup);
        $this->setNrestruturamrel($nrestruturamrel);
        $this->setDtinivigencia($dtinivigencia);
    }

}