<?php
/**
 * User: FelipeSantos
 * Date: 25/03/21
 * Time: 14:34
 * To change this template use File | Settings | File Templates.
 */

namespace HCM\Geral\Logic\Apolo;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Services\ESocialService;
use HCM\Util\DateUtil;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;

class InsercaoEsocial{
    
    private $hcmParameters;
    private $jenkins;
    private $entityManager;
    private $eSocialService;
    
    public function __construct(HCMParameters $hcmParameters, ESocialService $eSocialService){
        $this->entityManager = \Setforms2\Shared\Registry::getEntityManager();
        
        $this->hcmParameters = $hcmParameters;
        $this->eSocialService = $eSocialService;
        
        $this->jenkins = true;
        
        $this->envioCadastros = Repositories::ESO_ENVIOCADASTROS;
    }
    
    public function doInserts($parametros){
        
        set_time_limit(0);
        error_reporting(E_ERROR | E_PARSE);
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
        foreach ($parametros as $parametro) {
            $eventos = $this->entityManager->getRepository(Repositories::ESO_EVENTO)->getEventosESocial($parametro['DSVERSAO']);
            foreach ($eventos as $evento) {
                $row = array_merge($evento, $parametro);
                $this->setEnvioCadastros($row);
            }
        }
    }
    
    public function setEnvioCadastros($row) {
        try {
            
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            $nrtipoestrutlegal = $this->hcmParameters->getTipoEstruturaLegalOrg();
        
            $estruturaESocial = current($this->eSocialService->getEstruturasESocial(
                $nrorg, $dtmescompetenc, $nrtipoestrutlegal, $row['IDGRUPO'], 'AND NRESTRUTURAM = '.$row['NRESTRUTURAM']
            ));
            $row['CDCNPJEMPREGADOR'] = $estruturaESocial['CDCNPJEMPREGADOR'];
                
            $row['IDATIVO'] = isset($row['IDATIVO']) ? $row['IDATIVO'] : 'S';
            $row['DSDIRXMLENV'] = isset($row['DSDIRXMLENV']) ? $row['DSDIRXMLENV'] : null;
            $row['DSDIRXMLRETENV'] = isset($row['DSDIRXMLRETENV']) ? $row['DSDIRXMLRETENV'] : null;
            $row['DSDIRXMLRETPRO'] = isset($row['DSDIRXMLRETPRO']) ? $row['DSDIRXMLRETPRO'] : null;
            $row['DSDIRXMLRETTOT'] = isset($row['DSDIRXMLRETTOT']) ? $row['DSDIRXMLRETTOT'] : null;

            if ($row['NRTPAMBIENTE'] == '') {
                $paramESocial = $this->eSocialService->getParamESocial($row['NRESTRUTURAM']);
                if ($paramESocial) {
                    $row['NRTPAMBIENTE'] = $paramESocial['NRTPAMBIENTE'];
                }
            }
            
            if($row['NMEVPRECED']){
                $eventosPreced = explode(',', $row['NMEVPRECED']);
                foreach($eventosPreced as $eventoPreced){
                    $arrItens = $this->eSocialService->getItensParaEnvioEvento($eventoPreced, null, $row, $this->jenkins);
                    if ($arrItens) {
                        return;
                    }
                }
            }
            
            //verifica dependencia
            if($row['NMEVDEPEND']){
                $eventosDepend = explode(',', $row['NMEVDEPEND']);
                 foreach($eventosDepend as $eventoDepend){
                    $arrItens = $this->eSocialService->getItensParaEnvioEvento($eventoDepend, null, $row, $this->jenkins);
                    if (!$arrItens) {
                        return;
                    }
                }
            }
            
            echo '>>>Iniciando inserção [Org:' . $nrorg . '/Estrutura:' . $row['NRESTRUTURAM'] . '/Evento' . $row['NMEVENTO'] . ']<<<' . PHP_EOL;
            
            //Verifica itens para envio.
            $arrItens = $this->eSocialService->getItensParaEnvioEvento($row['NMEVENTO'], $row['IDGRUPO'], $row, $this->jenkins);
            if (empty($arrItens)) {
                echo '>>><<<' . PHP_EOL;
                return;
            }
            
            foreach($arrItens as $item) {
                $row['NRCHAVE'] = $item['NRCHAVE'];
                $row['NRTABELA'] = isset($item['NRTABELA']) ? $item['NRTABELA'] : null;
                $row['IDOPERACAO'] = isset($item['IDOPERACAO']) ? $item['IDOPERACAO'] : 'I';
                $row['DTMESCOMPETENC'] = $item['DTMESCOMPETENC'];
                $row['NRVINCULOM'] = array_key_exists('NRVINCULOM', $item) ? $item['NRVINCULOM'] : null;
                
                $erro = $this->eSocialService->beforeInsertEnvioCadastros($row['NRESTRUTURAM'], $row['NRTPAMBIENTE'], $row['NREVENTO'], $row['NRCHAVE'], $row['NRTABELA'], $row['IDOPERACAO'], $row['DTMESCOMPETENC'], $this->jenkins);
                
                if(!$erro){
                    $row['NRENVIO'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'ESO_ENVIOCADASTROS', 1, 12, true);
                    echo $row['NMEVENTO'] . ': Insert envio ' . $row['NRENVIO'] . ' chave ' . $item['NRCHAVE'] . PHP_EOL;
                    $this->insertEnvioCadastro($row);
                }
            }
            
            echo '>>><<<' . PHP_EOL;
        } catch(\Exception $e) {
            echo 'Erro setEnvioCadastros: ' . $e->getMessage() . PHP_EOL;
            echo '>>><<<' . PHP_EOL;
            return;
        }
    }
    
    public function insertEnvioCadastro($row){
        
        $envioCadastros = new $this->envioCadastros;
        $envioCadastros->setNrenvio($row['NRENVIO']);
        $envioCadastros->setNrchave($row['NRCHAVE']);
        $envioCadastros->setNrtabela($row['NRTABELA']);
        $envioCadastros->setIdoperacao($row['IDOPERACAO']);
        $envioCadastros->setDtmescompetenc(DateUtil::getDataDeString(substr($row['DTMESCOMPETENC'], 0, 10), DateUtil::FORMATO_BRASILEIRO, true));
        $envioCadastros->setNrevento($row['NREVENTO']);
        $envioCadastros->setNrestruturam($row['NRESTRUTURAM']);
        $envioCadastros->setNrtpambiente($row['NRTPAMBIENTE']);
        $envioCadastros->setNrvinculom($row['NRVINCULOM']);
        $envioCadastros->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $envioCadastros->setIdstatus('P');
        $envioCadastros->setDtinclusao(DateUtil::getDataAtual());
        $envioCadastros->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $envioCadastros->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $envioCadastros->setDtultatu(DateUtil::getDataAtual());
        $envioCadastros->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $envioCadastros->setCdoperultatu($this->hcmParameters->getCdoperador());
        
        $this->entityManager->persist($envioCadastros);
        $this->entityManager->flush($envioCadastros);
    }
}