<?php

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use HCM\Util\HCMParameters;
use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Geral\Services\HcmServices;
use Teknisa\Model\Gestaopessoas\Entities\GpePessoa;
use Teknisa\Model\Gestaopessoas\Entities\GpeVinculom;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DTO\Response\Error;

class IntegracaoDigitalCertisignService {
    /** @var EntityManager */
    protected $entityManager;

    /** @var HCMParameters */
    protected $hcmParameters;

    /** @var HcmServices */
    protected $hcmServices;

    public function __construct(
        EntityManager $entityManager,
        HCMParameters $hcmParameters,
        HcmServices   $hcmServices
    ) {
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->hcmServices   = $hcmServices;
    }

    public function saveAssinaturaEletronica(
        array $row
    ): void {
        $route = 'saveAssinaturaEletronica';
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();
        $cdOperador = $this->hcmParameters->getCdoperador();

        $nrVinculo = $row['NRVINCULOM'] ?? null;

        if (!empty($nrVinculo) && empty($row['NRPARCNEGOCIO'])) {

            /** @var GpeVinculom */
            $vinculo = $this->entityManager->
                getRepository(Repositories::GPE_VINCULOM)->
                findOneBy([
                    "nrvinculom" => $nrVinculo,
                    "nrorg" => $nrOrg,
                ]);

            $nrPessoa = $vinculo->getNrpessoa();

            /** @var GpePessoa */
            $pessoa = $this->entityManager->
                getRepository(Repositories::GPE_PESSOA)->
                findOneBy([
                    "nrpessoa" => $nrPessoa,
                    "nrorg" => $nrOrg,
                ]);

            $row['NRPARCNEGOCIO'] = $pessoa->getNrparcnegocio();
        }

        $row['CDOPERADOR'] = $cdOperador;

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];
        
        $this->hcmServices->send($route, $params, $nrOrg);
    }

    public function editAssinaturaEletronica(
        array $row
    ): void {
        $route = 'editAssinaturaEletronica';
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $this->hcmServices->send($route, $params, $nrOrg);
    }

    public function deleteAssinaturaEletronica(
        array $row
    ): void {
        $route = 'deleteAssinaturaEletronica';
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $this->hcmServices->send($route, $params, $nrOrg);
    }

    public function downloadAssinaturaEletronica(
        array $row
    ) {
        $route = 'downloadAssinaturaEletronica';
        $nrOrg = $this->hcmParameters->getNrorgTrabalho();

        $params = [
            'requestType' => 'Row',
            'row' => $row,
        ];

        $response = $this->hcmServices->send($route, $params, $nrOrg);
        $result = $response['dataset'][$route];

        return $result;
    }
}