<?php
/**
 * Created by PhpStorm.
 * User: leonardocarvalho
 * Date: 17/02/2016
 * Time: 10:51
 */

namespace HCM\Geral\Services;

use Doctrine\ORM\EntityManager;
use Geral\Entities\Itemprojimport;
use Geral\Entities\Logprojimport;
use HCM\Util\DateUtil;
use Importacao\Logic\Integracao\ProjetoImportacao;
use HCM\Util\HCMParameters;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\BTL\Repository\UploadsRepository;
use Zeedhi\Framework\Remote\FileUploader\Server;
use HCM\Util\Messages;
use Teknisa\Model\Gestaopessoas\Entities\GpeAnexos;
use HCM\Util\Repositories;

class UploadFileService {

    protected $entityManager;
    /** @var Environment */
    protected $hcmParameters;
    private $httpKernel;
    private $connection;
    protected $uploadsRepository;
    
    const IDDOCAPROVADO_AVAILABLE   = ['S', 'N'];
    const IDDOCAPROVADO_ONCREATE    = 'N';
    const IDDOCAPROVADO_APROVADO    = 'S';

    public function __construct(EntityManager $entityManager, HCMParameters $hcmParameters, Kernel $httpKernel, UploadsRepository $uploadsRepository, Server $zhFileUploader) {
        $this->entityManager     = $entityManager;
        $this->hcmParameters     = $hcmParameters;
        $this->httpKernel        = $httpKernel;
        $this->uploadsRepository = $uploadsRepository;
        $this->connection        = $this->entityManager->getConnection();
        $this->zhFileUploader    = $zhFileUploader;
        
        $this->gpeAnexos = Repositories::GPE_ANEXOS;
    }
    
    public function setGpeAnexos($dataset) {
        if (array_key_exists('DSPATH', $dataset)) {
            if (is_array($dataset['DSPATH'])) {

                $arquivos = $dataset['DSPATH'];
                foreach ($arquivos as $fileData) {
                    $type = null;
                    if(is_null($fileData["type"]) || $fileData["type"] == ''){
                        $type = explode('.', $fileData["name"])[1];
                    } else {
                        $type = $fileData["type"];
                    }
                    
                    $path = $fileData["path"];

                    // Gravação do logotipo da estrutura
                    if(isset($dataset['IDATUALIZALOGO'])) {
                    $this->setLogoEstrutura($path,$dataset);
                    }
                    

                    if (!isset($dataset['DSREFERENCIA']))
                        $dataset['DSREFERENCIA'] = $fileData['name'];
                        
                    $tipoDocumento  = $dataset['NRTIPODOCUMENTO']   ?? null;
                    $categDocumento = $dataset['NRCATEGDOCUMENTO']  ?? null;
                    
                    // var_dump($tipoDocumento, $categDocumento);die;

                    if (array_key_exists('NRANEXO', $dataset) && $dataset['NRANEXO'] != '' && $dataset['NRANEXO'] != null) {
                        
                        $file = $this->getGpeAnexo($dataset['NRANEXO'], $dataset['NRORG']);
                        if ($file) {
                            $this->uploadsRepository->updateGpeAnexos($dataset['NRANEXO'], $dataset['NRORG'], $dataset['NRPARCNEGOCIO'], $fileData["name"], $dataset['DSREFERENCIA'], $type, $dataset['CDOPERADOR_LOG'], $path, $dataset['IDFOTO']);
                        }
                    } else {
                        $nranexo = \HCM\Util\NovoCodigo::geraCodigo($dataset['NRORG'], 'GPE_ANEXOS', 1, 12, true);
                        if( !isset($dataset['IDDOCUMENTOMODELO']) ){
                            $dataset['IDDOCUMENTOMODELO'] = 'N';
                        }
                        $this->uploadsRepository->insertGpeAnexos($nranexo, $dataset['NRORG'], $dataset['NRPARCNEGOCIO'], $fileData["name"], $dataset['DSREFERENCIA'], $type, $dataset['CDOPERADOR_LOG'], $path, $dataset['IDFOTO'], $dataset['IDDOCUMENTOMODELO'], $tipoDocumento, $categDocumento);
                         
                    }
                    
                    // $this->entityManager->flush();
                }
            }
            return true;

        } else {
            return false;
        }
    }
    
    public function deleteUploadFile($filePath) {
        try{
            if(is_array($filePath))
                foreach($filePath as $val)
                    $this->zhFileUploader->deleteFile($val);
            else
                $this->zhFileUploader->deleteFile($filePath);
        } catch(\Exception $e) {
            throw $e;
        }
    }

    public function getGpeAnexo($nranexo, $nrorg) {
        $file = $this->uploadsRepository->getGpeAnexo($nranexo, $nrorg);
        if (count($file) > 0)
            return $file;
        else 
            return false;
    }
    
    public function aprovaStatusFile($nranexo) {
        $statusAprovado = self::IDDOCAPROVADO_APROVADO;
        $result = $this->alteraStatusGpeAnexo($nranexo, $statusAprovado);
        
        return $result;
    }
    
    /**
     *  Rotina responsável por aprovar/recusar documentos. Campo de aprovação: IDDOCAPROVADO
     * 
     *  @param  $nranexo
     *  @param  $status
     * 
     *  @return $result
     */
    public function alteraStatusGpeAnexo($nranexo, $status) {
        
        // Validação de status
        $statusInvalido = !in_array($status, self::IDDOCAPROVADO_AVAILABLE);
        if ($statusInvalido) {
            throw new \Exception(Messages::HCM00108_GER_STATUS_DOCUMENTO_INVALIDO);
        }
        
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $result = $this->uploadsRepository->updateIddocaprovadoGpeAnexo($nrorg, $nranexo, $status);
        
        return $result;
    }
    
     public function updateNomeGpeAnexos($row) {
        $nranexo = $row['NRANEXO'];
        $nrorg = $row['NRORG'];
        $nrparcnegocio = $row['NRPARCNEGOCIO'];
        $dsreferencia = $row['DSREFERENCIA'];
        return $this->uploadsRepository->updateNomeGpeAnexos($nranexo, $nrorg, $nrparcnegocio, $dsreferencia);
    }
    
    /*public function verificaLogo($row){
        $nrparcnegocio = $row['NRPARCNEGOCIO'];
        $nrorg = $row['NRORG'];
           if($nrparcnegocio){
                $nrparcnegocio = $this->entityManager->getRepository(Repositories::GPE_ANEXOS)->findOneBy(
                    array('nrorg' => $nrorg, 'nrparcnegocio' => $nrparcnegocio, 'iddocumentomodelo' => 'N', 'idfoto' => 'S'));
                if($nrparcnegocio != null) {
                    return "D";
                }else{
                $this->setGpeAnexos($row);
            }
        }
    
    }*/

    public function setLogoEstrutura($path, $dataset) {
       try {
        $nrestruturam = $dataset['NRESTRUTURAM'];
        $nrorg = $dataset['NRORG'];
        $pathLogoEstrutura = 'https://midia.teknisa.com/files/' . $path;
        $this->entityManager->beginTransaction();
        $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(
            array(
            'nrestruturam' =>$nrestruturam,
            'nrorg' => $nrorg
            ));
        
        $estruturam->setDsurllogoestrutura($pathLogoEstrutura);
        $this->entityManager->persist($estruturam);
        $this->entityManager->flush();
        $this->entityManager->commit();
       
        } catch(\Exception $e) {
        throw $e;
        }
    }

    public function deleteLogoEstrutura($row) {
        try {
            $this->entityManager->beginTransaction();
            $estruturam = $this->entityManager->getRepository(Repositories::ESTRUTURAM)->findOneBy(
            array(
            'nrparcnegocio' =>$row['NRPARCNEGOCIO'],
            'nrorg' => $row['NRORG']
            ));
            $estruturam->setDsurllogoestrutura(null);
            $this->entityManager->persist($estruturam);
            $this->entityManager->flush();
            $this->entityManager->commit();
        } catch(\Exception $e) {
            throw $e;
        }
    }
}