<?php

namespace HCM\Util\Mail;

use HCM\Util\Mail\Libraries\PHPMailer;

class SMTP {

    private $SMTPAuth    = true;
    private $IsHTML      = true;
    private $Host;
    private $Username;
    private $Password;
    private $From;
    private $FromName;
    private $NmAddress;
    private $Subject;
    private $NameBCC;
    private $NameCC;
    private $Attachments = array();
    private $SMTPSecure;
    
    public function __construct(array $params) {
        $this->Host      = $params['host'];
        $this->Username  = $params['username'];
        $this->Password  = $params['password'];
        $this->Port      = isset($params['port'])      ? $params['port']       : "5587";
        $this->From      = isset($params['from'])      ? $params['from']      : "no-reply@teknisa.com";
        $this->FromName  = isset($params['fromName'])  ? $params['fromName']  : "SAC Teknisa";
        $this->NmAddress = isset($params['nmAddress']) ? $params['nmAddress'] : "SAC Teknisa";
        $this->Subject   = isset($params['subject'])   ? $params['subject']   : "SAC Teknisa";
        $this->NameBCC   = isset($params['nameBCC'])   ? $params['nameBCC']   : "";
        $this->NameCC    = isset($params['nameCC'])    ? $params['nameCC']    : "";
        $this->SMTPSecure    = isset($params['SMTPSecure'])    ? $params['SMTPSecure']    : "";
        
    }

	public function sendMail($message, $address, $addressCC = '', $addressBCC = '') {
		$mail = new PHPMailer;
		$mail->CharSet  = 'UTF-8';
		$mail->IsHTML($this->IsHTML);
        $mail->isSMTP();                   // Set mailer to use SMTP
        $mail->Host     = $this->Host;     // Specify main and backup SMTP servers
        $mail->SMTPAuth = $this->SMTPAuth; // Enable SMTP authentication
        $mail->Username = $this->Username; // SMTP username
        $mail->Password = $this->Password; // SMTP password
        $mail->Port     = $this->Port;
        $mail->From     = $this->From;
        $mail->FromName = $this->FromName;
        $mail->Subject  = $this->Subject;
        $mail->Body     = $message;
        $mail->SMTPSecure = $this->SMTPSecure;
        
        if (is_array($address)) {
            foreach ($address as $to) {
                $mail->addAddress($to, $to); // Add multiple recipients
            }
        } else {
            $mail->addAddress($address, $this->NmAddress); // Add a recipient
        }

        //adiciona destinatários como cópia do email
        if (is_array($addressCC)) {
            foreach ($addressCC as $to) {
                $mail->AddCC($to, $to); // Add multiple recipients
            }
        } else {
            $mail->AddCC($addressCC, $this->NameCC); // Add a recipient
        }

        //adiciona destinatários como cópia oculta do email
        if (is_array($addressBCC)) {
            foreach ($addressBCC as $to) {
                $mail->AddBCC($to, $to); // Add multiple recipients
            }
        } else {
            $mail->AddBCC($addressBCC, $this->NameBCC); // Add a recipient
        }
        
        foreach ($this->Attachments as $attachment) { // Add all attachments
            $mail->addAttachment($attachment["path"], $attachment["name"], $attachment["encoding"], $attachment["type"], $attachment["disposition"]);
        }
        return $mail->send();
	}

    public function setNameBCC($NameBCC) {
		$this->NameBCC = $NameBCC;
    }

    public function setNameCC($NameCC) {
		$this->NameCC = $NameCC;
    }

    public function setHost($Host) {
		$this->Host = $Host;
    }

    public function setSMTPAuth($SMTPAuth) {
        $this->SMTPAuth = $SMTPAuth;
    }

    public function setUsername($Username) {
        $this->Username = $Username;
    }

    public function setIsHTML($IsHTML) {
        $this->IsHTML = $IsHTML;
    }

    public function setPassword($Password) {
        $this->Password = $Password;
    }

    public function setFrom($From) {
        $this->From = $From;
    }

    public function setFromName($FromName) {
        $this->FromName = $FromName;
    }

    public function setNmAddress($NmAddress) {
        $this->NmAddress = $NmAddress;
    }

    public function setSubject($Subject) {
        $this->Subject = $Subject;
    }

    public function addAttachment($path, $name = '', $encoding = 'base64', $type = '', $disposition = 'attachment') {
        $this->Attachments[] = array(
            "path"        => $path,
            "name"        => $name,
            "encoding"    => $encoding,
            "type"        => $type,
            "disposition" => $disposition
        );
    }
    
    public function clearAttachments(){
        $this->Attachments = array();
    }

}