<?php

namespace HCM\VAB\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;

class LinhatarifaController extends Crud {

    protected $dataSourceName = 'vab_linhatarifa';
    protected $dataSourceName1 =  'getLinhatarifa';
    protected $nmArquivoPlanilhaLT = 'LinhaTarifa';

    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, \HCM\VAB\Services\LinhatarifaService $linhatarifaService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->linhatarifaService = $linhatarifaService;
        parent::__construct($dataSourceManager);
    }

    public function getLinhatarifa (DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName1);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        $response->addDataSet($dataSet);
    }

    public function setLinhatarifa (DTO\Request\DataSet $request, DTO\Response $response) {
        foreach($request->getDataSet()->getRows() as &$row) {
            if(array_key_exists('NRLINHATARIFA', $row)){
                if($row['NRLINHATARIFA'] === '' ){
                    $row['NRLINHATARIFA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_LINHATARIFA', 1, 12, true);
                }
            } else {
                $row['NRLINHATARIFA'] = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_LINHATARIFA', 1, 12, true);
            }
        }

        parent::save($request, $response);
        return true;
    }

    public function delete(DTO\Request\DataSet $request, DTO\Response $response){
        parent::delete($request, $response);
    }
    
     public function baixarPlanilhaLinhatarifa(DTO\Request\DataSet $request, DTO\Response $response) {
        try{
            $file = $this->linhatarifaService->baixarPlanilhaLinhatarifa(
                $this->hcmParameters->getNrorgTrabalho(),
                $this->nmArquivoPlanilhaLT
            );
                
    
            if($file != null){
                $retorno = array('NMARQUIVO' => $file);
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $retorno));
            }
            
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );


        return $newFilterCriteria;
    }
    
    public function importarPlanilhaLinhatarifa(DTO\Request\DataSet $request, DTO\Response $response) {
        set_time_limit(0);
        try{
            $row = $request->getDataSet()->getRows();
            $row = $row[0];
            
            $messages = $this->linhatarifaService->importarPlanilhaLinhatarifa($row);
            if(empty($messages)){
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida com sucesso.", \Zeedhi\Framework\DTO\Response\Message::TYPE_MESSAGE));
            } else {
                $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message("Operação concluida. Existem erros na importação", \Zeedhi\Framework\DTO\Response\Message::TYPE_WARNING));
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msg', $messages));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}